#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>


#define F_CPU 8000000
#define AREF 5

#ifdef ADC_USE_IRQ
#define ADC_INIT_CHANNEL 0

int ADC_value[8];
ISR(TIMER2_COMPA_vect)
{
    // start an ADC conversion
    ADCSRA |= (1 << ADSC);
}

ISR(ADC_vect)
{
    // * store new result
    // * change channel
    // * start a new conversion if there are any channels left
    static uint8_t channel = ADC_INIT_CHANNEL;

    ADC_value[channel] = ADC;

    channel = (channel + 1) % 8;
    ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

    if (channel != ADC_INIT_CHANNEL)
        ADCSRA |= (1 << ADSC);
}

#endif

void ADC_init(void)
{
#ifndef ADC_USE_IRQ
	// enable ADC with:
	// *reference AVCC with external capacitor at AREF pin
	// *without left adjust of conversion result
	// *no auto-trigger
	// *no interrupt
	// *prescaler at 128
	ADMUX = (1 << REFS0);
	ADCSRA = (1 << ADEN) | (7 << ADPS0);
#else
	// enable ADC with interrupt
	ADMUX = (1 << REFS0) | (1 << 0) | (1 << 1)| (1 << 2) | (1 << 3);
	ADCSRA = (1 << ADEN) | (1 << ADSC) | (1 << ADIE) | (7 << ADPS0);
#endif
}

int ADC_get(uint8_t channel)
{
#ifndef ADC_USE_IRQ
	// start ADC conversion on "channel"
	// wait for completion
	// return the result
	ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

	ADCSRA |= (1 << ADSC);
	while(ADCSRA & (1 << ADSC));

	return ADC;
#else
	return ADC_value[channel];
#endif

    (void)channel;
    return 0;
}

/*
    Initializes the timer used for starting ADC conversions.
*/

void timer2_init()
{


	TCCR0A |= (1<<COM0A1) | (1<<COM0B1) | (1<<WGM01) | (1<<WGM00);

	/* set prescaler 1024, mode 7 (FAST PWM) */
	TCCR0B |=  (1<<WGM02) | (1<<CS02) | (1<<CS00);
	OCR0A=156;



	TCCR2A |= (1<<COM2A1) | (1<<COM2B1) | (1<<WGM21) | (1<<WGM20);

	/* set prescaler 1024, mode 7 (FAST PWM) */
	TCCR2B |=  (1<<WGM22) | (1<<CS22) | (1<<CS20);

    OCR2A=150;


	/* non intverted PWM */
	TCCR1A |= (1<<COM1A1) | (1<<COM1B1) | (1<<WGM11);

	/* set prescaler 1024, mode 14 (FAST PWM) */
	TCCR1B |= (1<<WGM13) | (1<<WGM12) | (1<<CS12) | (1<<CS10);

    ICR1=156;

}



int main()
{
	_delay_ms(2000);
	int lum0=0;
	ADC_init();
	DDRD |= (1 << PD6);
	DDRD |= (1<<PD5);
	DDRD |= (1<<PD4);
	DDRB |= (1<<PB4);
	DDRB |= (1<<PB7);

	timer2_init();

	/*
		Verific valoarea LDR daca este neagra misc servomotorul 
	*/
	while (1) {
		// servo 1
		lum0 = ADC_get(0);

		if(!(lum0 > 200)) {
			OCR0B = 35;
		} else {
			OCR0B = 40;
		}

		// servo 2
		lum0 = ADC_get(1);

		if(!(lum0 > 200)) {
			OCR1B = 20;
		} else {
			OCR1B = 25;
		}
		
		// servo 3
		lum0 = ADC_get(2);

		if(!(lum0 > 200)) {
			OCR1A = 25;
		} else {
			OCR1A = 20;
		}

		// servo 3
		lum0 = ADC_get(3);

		if(!(lum0 > 200)) {
			OCR2B = 80;
		} else {
			OCR2B = 50;
		}

		_delay_ms(10);

	}

	return 0;
	    
}
